@@ -154,14 +154,25 @@ class APIPluginConfigurationAdminSimulator(APIPluginConfigurationAdminContract):
 
         return self._data[plugin_name]
 
+    def create_or_update(self, plugin_name, plugin_configuration_id=None, enabled=None, consumer_id=None, **fields):
+        if plugin_configuration_id is not None:
+            current_plugin_name = None
+            for obj in self._data.values():
+                if obj['id'] == plugin_configuration_id:
+                    current_plugin_name = obj['name']
+                    break
+            assert current_plugin_name is not None
+            return self.update(current_plugin_name, enabled=enabled, consumer_id=consumer_id, **fields)
+        return self.create(plugin_name, enabled=enabled, consumer_id=consumer_id, **fields)
+
     def update(self, plugin_name, enabled=None, consumer_id=None, **fields):
         current_plugin_id = None
         current_plugin_name = None
 
-        for id in self._data:
-            if self._data[id]['name'] == plugin_name:
-                current_plugin_id = id
-                current_plugin_name = plugin_name
+        for obj in self._data.values():
+            if obj['name'] == plugin_name:
+                current_plugin_id = obj['id']
+                current_plugin_name = obj['name']
                 break
 
         if current_plugin_name is None or current_plugin_id is None:
@@ -185,9 +196,9 @@ class APIPluginConfigurationAdminSimulator(APIPluginConfigurationAdminContract):
         if enabled is not None and isinstance(enabled, bool):
             data_struct_update['enabled'] = enabled
 
-        self._data[current_plugin_id].update(data_struct_update)
+        self._data[current_plugin_name].update(data_struct_update)
 
-        return self._data[current_plugin_id]
+        return self._data[current_plugin_name]
 
     def list(self, size=100, offset=None, **filter_fields):
         data_list = [data_struct for data_struct in filter_dict_list(self._data.values(), **filter_fields)]
@@ -263,6 +274,20 @@ class APIAdminSimulator(APIAdminContract):
             'created_at': timestamp()
         }, check_conflict_keys=('name', 'public_dns'))
 
+    def add_or_update(self, target_url, api_id=None, name=None, public_dns=None, path=None, strip_path=False):
+        data = {
+            'name': name or public_dns,
+            'public_dns': public_dns,
+            'path': path,
+            'target_url': target_url,
+            'strip_path': strip_path
+        }
+
+        if api_id is not None:
+            return self.update(api_id, **data)
+
+        return self.add(**data)
+
     def update(self, name_or_id, target_url, **fields):
         # ensure trailing slash
         target_url = ensure_trailing_slash(target_url)
@@ -323,6 +348,17 @@ class ConsumerAdminSimulator(ConsumerAdminContract):
             'created_at': timestamp()
         }, check_conflict_keys=('username', 'custom_id'))
 
+    def create_or_update(self, consumer_id=None, username=None, custom_id=None):
+        data = {
+            'username': username,
+            'custom_id': custom_id
+        }
+
+        if consumer_id is not None:
+            return self.update(consumer_id, **data)
+
+        return self.create(**data)
+
     def update(self, username_or_id, **fields):
         return self._store.update(username_or_id, 'username', fields)
 
