@@ -134,6 +134,24 @@ class KongAdminTesting(object):
             self.assertEqual(result4['public_dns'], 'example.com')
             self.assertTrue(result4['strip_path'])
 
+        def test_add_or_update(self):
+            result = self.client.apis.add(target_url='http://mockbin.com', name='Mockbin', public_dns='mockbin.com')
+            self._cleanup_afterwards(result['id'])
+            self.assertEqual(self.client.apis.count(), 1)
+
+            # Test add_or_update without api_id -> Should ADD
+            result2 = self.client.apis.add_or_update(
+                target_url='http://mockbin2.com', name='Mockbin2', public_dns='mockbin2.com')
+            self._cleanup_afterwards(result2['id'])
+            self.assertEqual(self.client.apis.count(), 2)
+
+            # Test add_or_update with api_id -> Should UPDATE
+            result3 = self.client.apis.add_or_update(
+                target_url='http://mockbin3.com', api_id=result['id'], name='Mockbin3', public_dns='mockbin3.com')
+            self._cleanup_afterwards(result3['id'])
+            self.assertEqual(self.client.apis.count(), 2)
+            self.assertEqual(result3['id'], result['id'])
+
         def test_retrieve(self):
             result = self.client.apis.add(
                 target_url='http://mockbin.com', name=self._cleanup_afterwards('Mockbin'), public_dns='mockbin.com')
@@ -345,6 +363,30 @@ class KongAdminTesting(object):
             # Make sure we still have only 1 configuration
             self.assertEqual(self.client.apis.plugins('Mockbin').count(), 1)
 
+        def test_create_or_update_global_plugin_configuration(self):
+            # Create test api
+            result = self.client.apis.add(
+                target_url='http://mockbin.com', name=self._cleanup_afterwards('Mockbin'), public_dns='mockbin.com')
+            self.assertIsNotNone(result)
+            self.assertEqual(self.client.apis.plugins('Mockbin').count(), 0)
+
+            # Create global plugin configuration for the api
+            result2 = self.client.apis.plugins('Mockbin').create('ratelimiting', enabled=False, second=20)
+            self.assertIsNotNone(result2)
+            self.assertEqual(self.client.apis.plugins('Mockbin').count(), 1)
+
+            # Test create_or_update without plugin_id -> Should CREATE
+            result3 = self.client.apis.plugins('Mockbin').create_or_update(
+                'requestsizelimiting', enabled=True, allowed_payload_size=128)
+            self.assertIsNotNone(result3)
+            self.assertEqual(self.client.apis.plugins('Mockbin').count(), 2)
+
+            # Test create_or_update with plugin_configuration_id -> Should UPDATE
+            result4 = self.client.apis.plugins('Mockbin').create_or_update(
+                'requestsizelimiting', plugin_configuration_id=result3['id'], allowed_payload_size=512)
+            self.assertIsNotNone(result4)
+            self.assertEqual(self.client.apis.plugins('Mockbin').count(), 2)
+
         def test_update_incorrect_plugin_configuration(self):
             # Create test api
             result = self.client.apis.add(
@@ -476,6 +518,24 @@ class KongAdminTesting(object):
             self.assertEqual(result['username'], 'abc1234')
             self.assertEqual(result['custom_id'], '41245871-1s7q-awdd35aw-d8a6s2d12345')
 
+        def test_create_or_update(self):
+            result = self.client.consumers.create(username='abc1234', custom_id='41245871-1s7q-awdd35aw-d8a6s2d12345')
+            self._cleanup_afterwards(result['id'])
+            self.assertEqual(self.client.consumers.count(), 1)
+
+            # Test add_or_update without consumer_id -> Should CREATE
+            result2 = self.client.consumers.create_or_update(
+                username='abc12345', custom_id='41245871-1s7q-awdd35aw-d8a6s2d12346')
+            self._cleanup_afterwards(result2['id'])
+            self.assertEqual(self.client.consumers.count(), 2)
+
+            # Test add_or_update with consumer_id -> Should UPDATE
+            result3 = self.client.consumers.create_or_update(
+                consumer_id=result['id'], username='abc123456', custom_id='41245871-1s7q-awdd35aw-d8a6s2d12347')
+            self._cleanup_afterwards(result3['id'])
+            self.assertEqual(self.client.consumers.count(), 2)
+            self.assertEqual(result3['id'], result['id'])
+
         def test_create_only_username(self):
             result = self.client.consumers.create(username=self._cleanup_afterwards('abc123'))
             self.assertIsNotNone(result)
